
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 */


package cz.kitnarf.color;

import java.awt.Color;

/**
 *Versatile scale for filling zones by colors.
 *
 *@author Petr Frantík
 *@since 20130729
 *@version 20150702
 */
public class ColorZoner extends ColorFunction
{
	/** Array of interval borders.*/
	protected double[] borders;
	/** Array of color components.*/
	protected int[] components;




	/** {@inheritDoc} */
	@Override
	public Color getColor(double c)
	{
		int index=getBorderIndex(c);
		if(index<0) throw new IllegalArgumentException("Coordinate is outside zone borders.");
		//
		return getZoneColor(index);
	}

	/** {@inheritDoc} */
	@Override
	public Color getSafeColor(double c)
	{
		//System.out.print(c+" ");
		int index=getBorderIndex(c);
		if(index<0 || index>borders.length-2) return Color.WHITE;
		//
		return getZoneColor(index);
	}

	/** Gets color for an zone index. */
	public Color getZoneColor(int index)
	{
		if(index<0 || index>borders.length-2) throw new IllegalArgumentException("Bad zone index ("+index+").");
		//
		int e=index*3;
		int red=components[e++];
		int green=components[e++];
		int blue=components[e];
		//
		return new Color(red,green,blue);
	}


	//****************************************************************************
	/** Sets color components for each zone. */
	public void setComponents(int[] components){this.components=components;}
	/** Sets borders. */
	public void setBorders(double[] borders){this.borders=borders;}
	/** Gets number of borders. */
	public int getBorderCount(){return borders.length;}
	/** Gets a border. */
	public double getBorder(int index){return borders[index];}


	/** Finds and returns a border index according to coordinate or negative number if it is not found.*/
	public int getBorderIndex(double c)
	{
		int index=-1;
		for(int i=0;i<borders.length-1 && index<0;i++) if(c>=borders[i] && c<=borders[i+1]) index=i;
		return index;
	}

	/** Gets count of zones. */
	public int getZoneCount(){return getBorderCount()-1;}






	//*****************************************************************************************
	// MAIN
	//*****************************************************************************************
	/** Main method. */
	public static void main(String[] arguments) throws Exception
	{
		ColorZoner zoner=new ColorZoner();
		zoner.setBorders(new double[]{0,0.1,0.2,0.4,0.8,1.6,3.2});
		zoner.setComponents(new int[]{0,0,0,
																	50,50,50,
																	100,0,0,
																	0,100,0,
																	0,0,100,
																	100,100,0});
		System.out.println(zoner.getBorderIndex(3.21));
	}



}
