
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 */


package cz.kitnarf.color;

import java.awt.*;


//**************************************************************************************************************************
/**
 *Full-surface color function. This function is "spiral" on surface of RGB cube.
 *It has 390151 colors.
 *
 *@author Petr Frantík
 *@version 20130126
 */
public class FullSurfaceColorFunction extends ColorFunction
{



	//*****************************************************************************************
	/** {@inheritDoc} */
	@Override
	public Color getColor(double cd)
	{
		int[] ci=new int[3];
		int layer=0,layerSize=0,layerCoord=0;
		int coordLocal=0,sideSize=0,side=0,sideCoord=0,coordLocalSide=0;
		//
		//transformation / transformace
		int coord=(int)(cd*390151.9999999999); //celkem 390152 barev
		int mirrorCoord=coord;
		int part0Size=97921;
		int part01Size=194310;
		//
		//calculation / vypocet
		if(coord>0 && coord<part0Size)
		{
			layer=(int)((-1.5+Math.sqrt(2.25-6*(1-coord)))/3.0)+1;
			layerSize=3*layer;
			layerCoord=(int)(1.0+1.5*layer*(layer-1));
		}
		else if(coord<part0Size+part01Size)
		{
			layerSize=765;
			layer=(int)(256.0+(coord-part0Size)/layerSize);
			layerCoord=part0Size+layerSize*(layer-256);
		}
		else if(coord<390151)
		{
			mirrorCoord=390152-coord-1;
			//
			layer=(int)((-1.5+Math.sqrt(2.25-6*(1-mirrorCoord)))/3.0)+1;
			layerSize=3*layer;
			layerCoord=(int)(1.0+1.5*layer*(layer-1));
		}
		//
		//coordinations / souradnice
		if(coord>0 && coord<390151)
		{
			coordLocal=mirrorCoord-layerCoord;
			sideSize=(int)(layerSize/3.0);
			side=(int)(coordLocal/sideSize);
			sideCoord=side*sideSize;
			coordLocalSide=coordLocal-sideCoord;
		}
		//
		//color assignment / prirazeni barev
		if(coord==0)
		{
			ci[0]=0;
			ci[1]=0;
			ci[2]=0;
		}
		else if(coord==390151)
		{
			ci[0]=255;
			ci[1]=255;
			ci[2]=255;
		}
		else if(coord<part0Size)
		{
			ci[0]=sideSize-1-coordLocalSide;
			ci[1]=0;
			ci[2]=coordLocalSide+1;
		}
		else if(coord<part0Size+part01Size)
		{
			ci[0]=Math.min(255,sideSize-coordLocalSide+layer-256);
			ci[1]=Math.max(0,layer-256-coordLocalSide);
			ci[2]=coordLocalSide+1;
		}
		else
		{
			ci[0]=256-sideSize+coordLocalSide;
			ci[1]=254-coordLocalSide;
			ci[2]=255;
		}
		//
		//cube sides / strany krychle
		if(side==0) return new Color(ci[0],ci[1],ci[2]);
		else if(side==1) return new Color(ci[1],ci[2],ci[0]);
		else if(side==2) return new Color(ci[2],ci[0],ci[1]);
		else return new Color(128,128,128);
	}

}
