
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 */


package cz.kitnarf.color;

import java.awt.*;
import java.io.*;
import java.util.*;

import cz.kitnarf.util.PackageExplorer;


/**
 *Stores and provides list of clases extended from ColorFunction which were
 *detected from the package class structure or the JAR file.
 *
 *@author Petr Frantík
 *@version 20130126
 */
public class Repository
{
	private HashMap<String,Class> classMap;



	/** Finds and sets map of available color functions.*/
	public void setAvailableColorFunctions() throws ClassNotFoundException
	{
		java.util.List<Class> classes=findAvailableColorFunctions();
		classMap=new HashMap<String,Class>(classes.size());
		//
		for(Class c:classes)
		{
			classMap.put(c.getSimpleName(),c);
		}
	}




	/** Finds and gets list of available color functions.
	 *@return list of available color functions.*/
	public static java.util.List<Class> findAvailableColorFunctions() throws ClassNotFoundException
	{
		java.util.List<Class> allClasses=PackageExplorer.getClasses("cz.kitnarf.color");
		java.util.ArrayList<Class> subClasses=new java.util.ArrayList<Class>();
		//
		//odfiltrovani jinych nez potomku ColorFunction
		for(Class c:allClasses)
		{
			try
			{
				//if(c.getSuperclass().equals(ColorFunction.class)) subClasses.add(c);
				if(c.newInstance() instanceof ColorFunction) subClasses.add(c);
			}
			catch(InstantiationException ie)
			{
				//do nothing
			}
			catch(IllegalAccessException iae)
			{
				//do nothing
			}
 		}
		//
		return subClasses;
	}




	/** Gets array of available color function names.
	 *@param classes list of available classes.
	 *@return array of the available color function simple names.*/
	public static String[] getAvailableColorFunctionNames(java.util.List<Class> classes) throws ClassNotFoundException
	{
		String[] names=new String[classes.size()];
		//
		//ziskani simple names
		int i=0;
		for(Class c:classes)
		{
			names[i]=c.getSimpleName();
			i++;
		}
		//
		return names;
	}


	/** nGets set of available color function names from repository.
	 *@return set of available color function names.*/
	public Set<String> getAvailableColorFunctionNames() {return classMap.keySet();}


	/** Gets instance of the color function.
	 *@param name name of the color function.
	 *@return instance of the color function.*/
	public ColorFunction getColorFunctionInstance(String name) throws ClassNotFoundException,InstantiationException,IllegalAccessException
	{
		Class c=classMap.get(name);
		if(c==null) throw new ClassNotFoundException("Class named "+name+" not found in repository.");
		//
		Object o=c.newInstance();
		if(o instanceof ColorFunction) return (ColorFunction) o;
		else throw new ClassNotFoundException("Class named "+name+" is not instance of ColorFunction.");
	}





}
