
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 */


package cz.kitnarf.color;

import java.awt.*;


//**************************************************************************************************************************
/**
 *Septilinear color function. This function is composed from seven straight
 *segments. It has 1786 colors.
 *
 *@author Petr Frantík
 *@version 20130126
 */
public class SeptilinearColorFunction extends ColorFunction
{



	//*****************************************************************************************
	/** {@inheritDoc} */
	@Override
	public Color getColor(double cd)
	{
		int[] ci=new int[3];
		//
		//
		if(cd<=0.14285714285714)                     //0-B
		{
			ci[0]=0;
			ci[1]=0;
			ci[2]=(int)(cd*1786);
		}
		else if(cd<=0.28571428571428)                //B-R
		{
			ci[0]=(int)((cd-0.14285714285714)*1786+0.14285714285714);
			ci[1]=0;
			ci[2]=255-ci[0];
		}
		else if(cd<=0.42857142857142)               //R-M
		{
			ci[0]=255;
			ci[1]=0;
			ci[2]=(int)((cd-0.28571428571428)*1786+0.28571428571428);
		}
		else if(cd<=0.57142857142857)               //M-G
		{
			ci[0]=(int)((0.57142857142857-cd)*1786+0.42857142857142);
			ci[1]=(int)((cd-0.42857142857142)*1786+0.42857142857142);
			ci[2]=(int)((0.57142857142857-cd)*1786+0.42857142857142);
		}
		else if(cd<=0.71428571428571)               //G-C
		{
			ci[0]=0;
			ci[1]=255;
			ci[2]=(int)((cd-0.57142857142857)*1786+0.57142857142857);
		}
		else if(cd<=0.85714285714285)               //C-Y
		{
			ci[0]=(int)((cd-0.71428571428571)*1786+0.71428571428571);
			ci[1]=255;
			ci[2]=255-ci[0];
		}
		else                                              //Y-1
		{
			ci[0]=255;
			ci[1]=255;
			ci[2]=(int)((cd-0.85714285714285)*1786+0.8571428571);
		}
		//
		return new Color(ci[0],ci[1],ci[2]);
	}



	//*****************************************************************************************
	/**
	 *Calculate alernative color from color scale
	 *param cd coordinate
	 *return color from color scale
	 */
	public Color getAlternativeColor(double cd)
	{
		int[] ci=new int[3];
		//
		//
		if(cd<=0.14285714285714)                     //0-R
		{
			ci[0]=(int)(cd*1786);
			ci[1]=0;
			ci[2]=0;
		}
		else if(cd<=0.28571428571428)                //R-M
		{
			ci[0]=255;
			ci[1]=0;
			ci[2]=(int)((cd-0.14285714285714)*1786+0.14285714285714);
		}
		else if(cd<=0.42857142857142)               //M-B
		{
			ci[0]=(int)((0.42857142857142-cd)*1786+0.57142857142857);
			ci[1]=0;
			ci[2]=255;
		}
		else if(cd<=0.57142857142857)               //B-C
		{
			ci[0]=0;
			ci[1]=(int)((cd-0.42857142857142)*1786+0.42857142857142);
			ci[2]=255;
		}
		else if(cd<=0.71428571428571)               //C-G
		{
			ci[0]=0;
			ci[1]=255;
			ci[2]=(int)((0.71428571428571-cd)*1786+0.28571428571428);
		}
		else if(cd<=0.85714285714285)               //G-Y
		{
			ci[0]=(int)((cd-0.71428571428571)*1786+0.71428571428571);
			ci[1]=255;
			ci[2]=0;
		}
		else                                              //Y-1
		{
			ci[0]=255;
			ci[1]=255;
			ci[2]=(int)((cd-0.85714285714285)*1786+0.8571428571);
		}
		//
		return new Color(ci[0],ci[1],ci[2]);
	}

}
