
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 */


package cz.kitnarf.color;

import java.awt.*;


//**************************************************************************************************************************
/**
 *Spiral color function. This function is composed from two segments of spirals
 *in RGB space. It has 3172 colors.
 *
 *@author Petr Frantík
 *@version 20130126
 */
public class SpiralColorFunction extends ColorFunction
{



	//*****************************************************************************************
	/** {@inheritDoc} */
	@Override
	public Color getColor(double cd)
	{
		int[] ci=new int[3];
		//
		//transformation / transformace
		cd*=255.999999999999;
		//
		//parameters / parametry
		double amplitude=0.70710678118654752440084436210485; //1.0/Math.sqrt(2)
		double frequency=8.0*Math.PI/256;
		double pitch=1.7320508075688772935274463415059;  //Math.sqrt(3)  ...stoupani
		//
		//amplitude / amplituda
		double a=amplitude*pitch;
		if(cd<128) a*=cd;
		else a*=255.999999999999-cd;
		//
		//spiral axis z
		double x0=a*Math.cos(cd*frequency);
		double y0=a*Math.sin(cd*frequency);
		double z0=cd*pitch;
		//
		//x axis rotation / rotace kolem x
		double cos=0.57735026918962576450914878050196;
		double sin=-0.81649658092772603273242802490196;
		double x1=x0;
		double y1=y0*cos-z0*sin;
		double z1=y0*sin+z0*cos;
		//
		//z axis rotation / rotace kolem z
		cos=0.70710678118654752440084436210485;
		sin=-cos;
		double x2=x1*cos-y1*sin;
		double y2=x1*sin+y1*cos;
		double z2=z1;
		//
		if(x2>255) x2=255;
		if(y2>255) y2=255;
		if(z2>255) z2=255;
		//
		return new Color((int)x2,(int)y2,(int)z2);
	}

}
